/* ScrutariDataURI - Copyright (c) 2005-2016 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.scrutari.datauri;

import java.net.URI;
import java.net.URISyntaxException;


/**
 *
 * @author Vincent Calame
 */
public class URIParser {

    private URIParser() {
    }

    public static ScrutariDataURI parse(String s) throws URIParseException {
        try {
            URI uri = new URI(s);
            String scheme = uri.getScheme();
            if (scheme == null) {
                throw new URIParseException(URIParseException.MISSINGSCHEME_EXCEPTION, "missing scheme");
            }
            if (uri.isOpaque()) {
                throw new URIParseException(URIParseException.OPAQUEURI_EXCEPTION, "opaque uri");
            }
            String path = uri.getRawPath();
            if (path == null) {
                throw new URIParseException(URIParseException.MALFORMEDPATH_EXCEPTION, "raw path is null");
            }
            String tokens[] = path.split("/");
            int size = tokens.length - 1;
            if (size < 2) {
                throw new URIParseException(URIParseException.MALFORMEDPATH_EXCEPTION, "missing parts");
            }
            if (size > 4) {
                throw new URIParseException(URIParseException.MALFORMEDPATH_EXCEPTION, "too many parts");
            }
            if (scheme.equals("fiche")) {
                if (size != 4) {
                    throw new URIParseException(URIParseException.MALFORMEDPATH_EXCEPTION, "need 4 parts");
                }
                return new FicheURI(tokens[1], tokens[2], tokens[3], tokens[4]);
            } else if (scheme.equals("corpus")) {
                if (size != 3) {
                    throw new URIParseException(URIParseException.MALFORMEDPATH_EXCEPTION, "need 3 parts");
                }
                return new CorpusURI(tokens[1], tokens[2], tokens[3]);
            } else if (scheme.equals("thesaurus")) {
                if (size != 3) {
                    throw new URIParseException(URIParseException.MALFORMEDPATH_EXCEPTION, "need 3 parts");
                }
                return new ThesaurusURI(tokens[1], tokens[2], tokens[3]);
            } else if (scheme.equals("motcle")) {
                if (size != 4) {
                    throw new URIParseException(URIParseException.MALFORMEDPATH_EXCEPTION, "need 4 parts");
                }
                return new MotcleURI(tokens[1], tokens[2], tokens[3], tokens[4]);
            } else if (scheme.equals("base")) {
                if (size != 2) {
                    throw new URIParseException(URIParseException.MALFORMEDPATH_EXCEPTION, "need 2 parts");
                }
                return new BaseURI(tokens[1], tokens[2]);
            } else {
                throw new URIParseException(URIParseException.UNKNOWNSCHEME_EXCEPTION, "unknown scheme: " + scheme);
            }
        } catch (URISyntaxException urise) {
            throw new URIParseException(URIParseException.URISYNTAX_EXCEPTION, urise.getMessage());
        }
    }

}
