/* ScrutariLib_DB - Copyright (c) 2014 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */
package net.scrutari.db.tools;

import java.util.HashMap;
import java.util.Map;
import net.scrutari.datauri.ScrutariDataURI;
import net.scrutari.db.api.DataCoder;
import net.scrutari.db.api.HistoryDataCoder;

/**
 *
 * @author Vincent Calame
 */
public final class DataCoderUtils {

    private DataCoderUtils() {

    }

    /**
     * L'instance de DataCoder est encaspsulé dans une nouvelle instance.
     * getCode s'appuie sur historyDataCoder.getCode mais checkCode ne conserve que
 les codes réelement demandé et non tout ceux contenu dans l'argument.
     */
    public static DataCoder encapsulate(HistoryDataCoder historyDataCoder) {
        return new InternalDataCoder(historyDataCoder);
    }

    private static class InternalDataCoder implements DataCoder {

        private final HistoryDataCoder historyDataCoder;
        private final Map<ScrutariDataURI, Integer> codeByScrutariDataURIMap = new HashMap<ScrutariDataURI, Integer>();

        private InternalDataCoder(HistoryDataCoder historyDataCoder) {
            this.historyDataCoder = historyDataCoder;
        }

        @Override
        public int getCode(ScrutariDataURI scrutariDataURI) {
            Integer itg = codeByScrutariDataURIMap.get(scrutariDataURI);
            if (itg != null) {
                return itg;
            }
            int code = historyDataCoder.getCode(scrutariDataURI);
            codeByScrutariDataURIMap.put(scrutariDataURI, code);
            return code;
        }

        @Override
        public int checkCode(ScrutariDataURI scrutariDataURI) {
            Integer itg = codeByScrutariDataURIMap.get(scrutariDataURI);
            if (itg == null) {
                return -1;
            }
            return itg;
        }

    }

}
