/* ScrutariLib_DB - Copyright (c) 2016 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.scrutari.db.tools.codes;

import java.util.HashMap;
import java.util.LinkedHashMap;
import java.util.Map;
import net.scrutari.datauri.ScrutariDataURI;
import net.scrutari.db.api.codes.ScrutariDataURICodeList;


/**
 *
 * @author Vincent Calame
 */
public class ScrutariDataCodeListBuilder {

    private final Map<Integer, ScrutariDataURI> codeMap = new LinkedHashMap<Integer, ScrutariDataURI>();
    private final Map<ScrutariDataURI, Integer> uriMap = new HashMap<ScrutariDataURI, Integer>();

    public ScrutariDataCodeListBuilder() {

    }

    public void addScrutariDataURI(int code, ScrutariDataURI scrutariDataURI) {
        if (codeMap.containsKey(code)) {
            return;
        }
        if (uriMap.containsKey(scrutariDataURI)) {
            return;
        }
        codeMap.put(code, scrutariDataURI);
        uriMap.put(scrutariDataURI, code);
    }

    public boolean containsCode(int code) {
        return codeMap.containsKey(code);
    }

    public boolean containsURI(ScrutariDataURI scrutariDataURI) {
        return uriMap.containsKey(scrutariDataURI);
    }

    public int size() {
        return codeMap.size();
    }

    public ScrutariDataURICodeList toScrutariDataURICodeList() {
        int size = codeMap.size();
        int[] codeArray = new int[size];
        ScrutariDataURI[] uriArray = new ScrutariDataURI[size];
        int p = 0;
        for (Map.Entry<Integer, ScrutariDataURI> entry : codeMap.entrySet()) {
            codeArray[p] = entry.getKey();
            uriArray[p] = entry.getValue();
            p++;
        }
        return new InternalScrutariDataURICodeList(codeArray, uriArray);
    }


    private static class InternalScrutariDataURICodeList implements ScrutariDataURICodeList {

        private final int[] codeArray;
        private final ScrutariDataURI[] uriArray;

        public InternalScrutariDataURICodeList(int[] codeArray, ScrutariDataURI[] uriArray) {
            this.codeArray = codeArray;
            this.uriArray = uriArray;
        }

        @Override
        public int getCodeCount() {
            return codeArray.length;
        }

        @Override
        public int getCode(int i) {
            return codeArray[i];
        }

        @Override
        public ScrutariDataURI getScrutariDataURI(int i) {
            return uriArray[i];
        }

    }

}
