/* ScrutariLib_DB - Copyright (c) 2014 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.scrutari.db.tools.stats;

import java.util.HashMap;
import java.util.Map;
import java.util.SortedMap;
import java.util.TreeMap;
import net.mapeadores.util.localisation.Lang;
import net.scrutari.db.api.stats.FicheStats;


/**
 *
 * @author Vincent Calame
 */
public class FicheStatsBuilder {

    public final static FicheStats EMPTY_FICHESTATS = new InternalFicheStats();
    private final Map<Lang, Counter> counterMap = new HashMap<Lang, Counter>();

    public FicheStatsBuilder() {

    }

    public void addOne(Lang lang) {
        Counter counter = counterMap.get(lang);
        if (counter == null) {
            counter = new Counter(lang);
            counterMap.put(lang, counter);
        }
        counter.increase(1);
    }

    public void addFicheStats(FicheStats ficheStats) {
        int langCount = ficheStats.getLangCount();
        for (int i = 0; i < langCount; i++) {
            Lang langInteger = ficheStats.getLang(i);
            Counter counter = counterMap.get(langInteger);
            if (counter == null) {
                counter = new Counter(langInteger);
                counterMap.put(langInteger, counter);
            }
            counter.increase(ficheStats.getFicheCount(i));
        }
    }

    public FicheStats toFicheStats() {
        SortedMap<Lang, Integer> langMap = new TreeMap<Lang, Integer>();
        for (Counter counter : counterMap.values()) {
            langMap.put(counter.lang, counter.value);
        }
        int size = langMap.size();
        Lang[] langArray = new Lang[size];
        int[] ficheCount = new int[size];
        int p = 0;
        for (Map.Entry<Lang, Integer> entry : langMap.entrySet()) {
            langArray[p] = entry.getKey();
            ficheCount[p] = entry.getValue();
            p++;
        }
        return new InternalFicheStats(langArray, ficheCount);
    }


    private static class Counter {

        private final Lang lang;
        private int value = 0;

        private Counter(Lang lang) {
            this.lang = lang;
        }

        private void increase(int plus) {
            value = value + plus;
        }

        private int getCount() {
            return value;
        }

    }


    private static class InternalFicheStats implements FicheStats {

        private final Lang[] langArray;
        private final int[] countArray;

        private InternalFicheStats(Lang[] langArray, int[] countArray) {
            this.langArray = langArray;
            this.countArray = countArray;
        }

        private InternalFicheStats() {
            this.langArray = new Lang[0];
            this.countArray = new int[0];
        }

        @Override
        public int getLangCount() {
            return langArray.length;
        }

        @Override
        public Lang getLang(int index) {
            return langArray[index];
        }

        @Override
        public int getFicheCount(int index) {
            return countArray[index];
        }

    }


}
