/* ScrutariLib_Lexie - Copyright (c) 2011-2012 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.scrutari.lexie;

import java.io.IOException;
import net.mapeadores.util.primitives.io.PrimitivesReader;
import net.mapeadores.util.primitives.io.PrimitivesWriter;
import net.mapeadores.util.text.collation.CollationUnitPosition;


/**
 *
 * @author Vincent Calame
 */
public class LexieOccurrences {

    private int code;
    private int[] alineaIntegerArray;
    private CollationUnitPosition[] bestPositionInAlineaArray;
    private int[] alineaWholeLengthArray;
    private int[][] lexieIndexInLexificationArray;

    public LexieOccurrences() {
    }

    public int getCode() {
        return code;
    }

    public void setCode(int code) {
        this.code = code;
    }

    public void setArrays(int[] alineaIntegerArray, CollationUnitPosition[] bestPositionInAlineaArray, int[] alineaWholeLengthArray, int[][] lexieIndexInLexificationArray) {
        this.alineaIntegerArray = alineaIntegerArray;
        this.bestPositionInAlineaArray = bestPositionInAlineaArray;
        this.alineaWholeLengthArray = alineaWholeLengthArray;
        this.lexieIndexInLexificationArray = lexieIndexInLexificationArray;
    }

    public int getAlineaCount() {
        return alineaIntegerArray.length;
    }

    public int getAlineaInteger(int i) {
        return alineaIntegerArray[i];
    }

    public CollationUnitPosition getBestPositionInAlinea(int i) {
        return bestPositionInAlineaArray[i];
    }

    public int getAlineaWholeLength(int i) {
        return alineaWholeLengthArray[i];
    }

    public int[] getLexieIndexInFieldLexification(int i) {
        return lexieIndexInLexificationArray[i];
    }

    public static LexieOccurrences fromPrimitives(PrimitivesReader primitivesReader) throws IOException {
        LexieOccurrences lexieOccurrences = new LexieOccurrences();
        lexieOccurrences.setCode(primitivesReader.readInt());
        int length = primitivesReader.readInt();
        int[] fieldNumberArray = new int[length];
        CollationUnitPosition[] bestPositionInfieldArray = new CollationUnitPosition[length];
        int[] fieldWholeLengthArray = new int[length];
        int[][] lexieIndexInLexificationArray = new int[length][];
        for (int i = 0; i < length; i++) {
            fieldNumberArray[i] = primitivesReader.readInt();
            bestPositionInfieldArray[i] = LexieUtils.readCollationUnitPosition(primitivesReader);
            fieldWholeLengthArray[i] = primitivesReader.readInt();
            int ln2 = primitivesReader.readInt();
            int[] array = new int[ln2];
            for (int j = 0; j < ln2; j++) {
                array[j] = primitivesReader.readInt();
            }
            lexieIndexInLexificationArray[i] = array;
        }
        lexieOccurrences.setArrays(fieldNumberArray, bestPositionInfieldArray, fieldWholeLengthArray, lexieIndexInLexificationArray);
        return lexieOccurrences;
    }

    public static void toPrimitives(LexieOccurrences lexieOccurrences, PrimitivesWriter primitivesWriter) throws IOException {
        primitivesWriter.writeInt(lexieOccurrences.getCode());
        int length = lexieOccurrences.alineaIntegerArray.length;
        primitivesWriter.writeInt(length);
        for (int i = 0; i < length; i++) {
            primitivesWriter.writeInt(lexieOccurrences.getAlineaInteger(i));
            LexieUtils.writeCollationUnitPosition(lexieOccurrences.getBestPositionInAlinea(i), primitivesWriter);
            primitivesWriter.writeInt(lexieOccurrences.getAlineaWholeLength(i));
            int[] array = lexieOccurrences.getLexieIndexInFieldLexification(i);
            int ln2 = array.length;
            primitivesWriter.writeInt(ln2);
            for (int j = 0; j < ln2; j++) {
                primitivesWriter.writeInt(array[j]);
            }
        }
    }

}
