/* ScrutariLib_SearchEngine - Copyright (c) 2015-2016 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.scrutari.searchengine.tools.operands.filters;

import java.text.ParseException;
import net.mapeadores.util.localisation.Lang;
import net.mapeadores.util.logicaloperation.SimpleOperand;
import net.scrutari.data.FicheData;
import net.scrutari.searchengine.api.operands.OperandMessageKeys;
import net.scrutari.searchengine.api.operands.OperandsConstants;
import net.scrutari.searchengine.api.operands.filters.FicheDataOperand;
import net.scrutari.searchengine.tools.OperandCheck;


/**
 *
 * @author Vincent Calame
 */
final class LangCheck {

    private LangCheck() {

    }

    static OperandCheck check(SimpleOperand simpleOperand, OperandConverterParameters operandConverterParameters) {
        FicheDataOperand operand = toOperand(simpleOperand, operandConverterParameters);
        if (operand == null) {
            return OperandCheck.UNDEFINED_OPERANDECHECK;
        }
        return new OperandCheck(null, operand);
    }

    private static FicheDataOperand toOperand(SimpleOperand simpleOperand, OperandConverterParameters operandConverterParameters) {
        boolean acceptMode = simpleOperand.isAcceptMode();
        String operandBody = simpleOperand.getBody();
        try {
            Lang lang = Lang.parse(operandBody);
            return new LangOperand(lang, acceptMode, operandBody);
        } catch (ParseException pe) {
            operandConverterParameters.addError(OperandMessageKeys.WRONG_LANG_OPERAND, simpleOperand.getOperandString());
            return null;
        }
    }


    private static class LangOperand implements FicheDataOperand {

        private final boolean acceptMode;
        private final Lang lang;
        private final String operandBody;

        private LangOperand(Lang lang, boolean acceptMode, String operandBody) {
            this.lang = lang;
            this.acceptMode = acceptMode;
            this.operandBody = operandBody;
        }

        @Override
        public boolean acceptFicheData(FicheData ficheData) {
            if (ficheData == null) {
                return !acceptMode;
            }
            boolean test = ficheData.getLang().equals(lang);
            if (!acceptMode) {
                test = !test;
            }
            return test;
        }

        @Override
        public String toString() {
            if (acceptMode) {
                return OperandsConstants.LANG_TYPE + ":" + operandBody;
            } else {
                return "!" + OperandsConstants.LANG_TYPE + ":" + operandBody;
            }
        }

    }

}
