/* ScrutariLib_SearchEngine - Copyright (c) 2013-2016 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.scrutari.searchengine.tools.operands.filters;

import java.util.HashSet;
import java.util.Set;
import net.mapeadores.util.logicaloperation.SimpleOperand;
import net.scrutari.datauri.ScrutariDataURI;
import net.scrutari.db.api.DataAccess;
import net.scrutari.db.api.codes.CodeList;
import net.scrutari.db.api.codes.IndexationCodeList;
import net.scrutari.db.tools.codes.CodeParser;
import net.scrutari.searchengine.api.operands.OperandsConstants;
import net.scrutari.searchengine.api.operands.filters.CodeOperand;
import net.scrutari.searchengine.tools.OperandCheck;


/**
 *
 * @author Vincent Calame
 */
final class MotcleCheck {

    private MotcleCheck() {
    }

    static OperandCheck check(SimpleOperand simpleOperand, CodeList corpusCodeList, OperandConverterParameters operandConverterParameters) {
        boolean acceptMode = simpleOperand.isAcceptMode();
        String operandBody = simpleOperand.getBody();
        DataAccess dataAccess = operandConverterParameters.getDataAccess();
        int codemotcle = CodeParser.toCode(dataAccess, operandBody, operandConverterParameters.getMessageHandler(), ScrutariDataURI.MOTCLEURI_TYPE);
        if (codemotcle == -1) {
            return OperandCheck.UNDEFINED_OPERANDECHECK;
        }
        Set<Integer> corpusSet = new HashSet<Integer>();
        Set<Integer> ficheSet = new HashSet<Integer>();
        int corpusCount = corpusCodeList.getCodeCount();
        for (int j = 0; j < corpusCount; j++) {
            int codecorpus = corpusCodeList.getCode(j);
            IndexationCodeList indexationCodeList = dataAccess.getIndexationCodeList(codemotcle, codecorpus);
            int ficheCount = indexationCodeList.getCodeCount();
            if (ficheCount > 0) {
                for (int k = 0; k < ficheCount; k++) {
                    int codefiche = indexationCodeList.getCode(k);
                    ficheSet.add(codefiche);
                }
                corpusSet.add(codecorpus);
            }
        }
        if (!acceptMode) {
            corpusSet = null;
        }
        return new OperandCheck(corpusSet, new MotcleOperand(ficheSet, acceptMode, operandBody));
    }


    private static class MotcleOperand implements CodeOperand {

        private final Set<Integer> codeficheSet;
        private final boolean acceptMode;
        private final String operandBody;

        private MotcleOperand(Set<Integer> codeficheSet, boolean acceptMode, String operandBody) {
            this.codeficheSet = codeficheSet;
            this.acceptMode = acceptMode;
            this.operandBody = operandBody;
        }

        @Override
        public boolean acceptCode(int code) {
            boolean test = codeficheSet.contains(code);
            if (!acceptMode) {
                test = !test;
            }
            return test;
        }

        @Override
        public String toString() {
            if (acceptMode) {
                return OperandsConstants.MOTCLE_TYPE + ":" + operandBody;
            } else {
                return "!" + OperandsConstants.MOTCLE_TYPE + ":" + operandBody;
            }
        }

    }

}
