/* ScrutariLib_SearchEngine - Copyright (c) 2014-2016 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.scrutari.searchengine.tools.options;

import java.util.ArrayList;
import java.util.List;
import net.mapeadores.util.attr.AttributeHolder;
import net.mapeadores.util.localisation.Lang;
import net.mapeadores.util.misc.DefBuilder;
import net.mapeadores.util.text.Label;
import net.mapeadores.util.text.LabelHolder;
import net.scrutari.datauri.CorpusURI;
import net.scrutari.searchengine.api.options.CategoryDef;


/**
 *
 * @author Vincent Calame
 */
public class CategoryDefBuilder extends DefBuilder {

    private final String name;
    private final List<CorpusURI> uriList = new ArrayList<CorpusURI>();
    private int rank;

    public CategoryDefBuilder(String name) {
        this.name = name;
    }

    public void addCorpusURI(CorpusURI corpusURI) {
        uriList.add(corpusURI);
    }

    public void setRank(int rank) {
        this.rank = rank;
    }

    public CategoryDef toCategoryDef() {
        CorpusURI[] corpusURIArray = uriList.toArray(new CorpusURI[uriList.size()]);
        LabelHolder labelHolder = toLabelHolder();
        AttributeHolder attributeHolder = toAttributeHolder();
        return new InternalCategoryDef(name, rank, corpusURIArray, labelHolder, attributeHolder);
    }


    private static class InternalCategoryDef extends DefBuilder.Def implements CategoryDef {

        private final String name;
        private final int rank;
        private final CorpusURI[] corpusURIArray;
        private final LabelHolder libelleHolder;


        private InternalCategoryDef(String name, int rank, CorpusURI[] corpusURIArray,
                LabelHolder libelleHolder, AttributeHolder attributeHolder) {
            super(libelleHolder, attributeHolder);
            this.name = name;
            this.rank = rank;
            this.corpusURIArray = corpusURIArray;
            this.libelleHolder = libelleHolder;
        }

        @Override
        public String getName() {
            return name;
        }

        @Override
        public int getCorpusURICount() {
            return corpusURIArray.length;
        }

        @Override
        public CorpusURI getCorpusURI(int i) {
            return corpusURIArray[i];
        }

        @Override
        public int getRank() {
            return rank;
        }

        @Override
        public int getLabelCount() {
            return libelleHolder.getLabelCount();
        }

        @Override
        public Label getLabelByIndex(int i) {
            return libelleHolder.getLabelByIndex(i);
        }

        @Override
        public Label getLabelByLang(Lang lang) {
            return libelleHolder.getLabelByLang(lang);
        }

    }

}
