/* ScrutariLib_SearchEngine - Copyright (c) 2011-2014 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.scrutari.searchengine.tools.result;

import net.mapeadores.util.localisation.Lang;
import net.scrutari.searchengine.api.result.InLangOccurrenceList;
import net.scrutari.searchengine.api.result.MotcleSearchResultInfo;


/**
 *
 * @author Vincent Calame
 */
public class MotcleSearchResultBuilder {

    private InternalMotcleSearchResultInfo motcleSearchResultInfo = new InternalMotcleSearchResultInfo();

    public MotcleSearchResultBuilder() {
    }

    public void setCode(int code) {
        motcleSearchResultInfo.code = code;
    }

    public void setInLangOccurrenceListArray(InLangOccurrenceList[] inLangOccurrenceListArray) {
        motcleSearchResultInfo.inLangOccurrenceListArray = inLangOccurrenceListArray;
    }

    public MotcleSearchResultInfo toMotcleSearchResultInfo() {
        return motcleSearchResultInfo;
    }

    public static MotcleSearchResultInfo toMotcleSearchResultInfo(int code, InLangOccurrenceList[] inLangOccurrenceListArray) {
        return new InternalMotcleSearchResultInfo(code, inLangOccurrenceListArray);
    }


    private static class InternalMotcleSearchResultInfo implements MotcleSearchResultInfo {

        private int code;
        private InLangOccurrenceList[] inLangOccurrenceListArray;

        private InternalMotcleSearchResultInfo() {
        }

        private InternalMotcleSearchResultInfo(int code, InLangOccurrenceList[] inLangOccurrenceListArray) {
            this.code = code;
            this.inLangOccurrenceListArray = inLangOccurrenceListArray;
        }

        @Override
        public int getCode() {
            return code;
        }

        @Override
        public int getInLangOccurrenceListCount() {
            if (inLangOccurrenceListArray == null) {
                return 0;
            }
            return inLangOccurrenceListArray.length;
        }

        @Override
        public InLangOccurrenceList getInLangOccurrenceList(int i) {
            return inLangOccurrenceListArray[i];
        }

        @Override
        public InLangOccurrenceList getInLangOccurrenceListByLang(Lang lang) {
            if (inLangOccurrenceListArray == null) {
                return null;
            }
            int length = inLangOccurrenceListArray.length;
            for (int i = 0; i < length; i++) {
                InLangOccurrenceList inLangOccurrenceList = inLangOccurrenceListArray[i];
                if (inLangOccurrenceList.getLang().equals(lang)) {
                    return inLangOccurrenceList;
                }
            }
            return null;
        }

    }

}
