/* SctServer - Copyright (c) 2011-2014 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.sctserver.request.json;

import java.io.IOException;
import net.mapeadores.util.exceptions.ShouldNotOccurException;
import net.mapeadores.util.json.JSONException;
import net.mapeadores.util.json.JSONWriter;
import net.mapeadores.util.localisation.Message;
import net.mapeadores.util.localisation.MessageList;
import net.mapeadores.util.logging.CommandMessage;


/**
 *
 * @author Vincent Calame
 */
public class ErrorJsonProducer extends AbstractJsonProducer {

    private final CommandMessage commandMessage;

    public ErrorJsonProducer(CommandMessage commandMessage) {
        this.commandMessage = commandMessage;
    }

    @Override
    public void writeJson(Appendable appendable) throws IOException {
        String errorKey = commandMessage.getMessageKey();
        Object valueObj = commandMessage.getMessageValue();
        String parameter;
        String value = null;
        if (valueObj.getClass().isArray()) {
            Object[] objs = (Object[]) valueObj;
            int length = objs.length;
            if (length == 0) {
                parameter = "";
            } else {
                parameter = objs[0].toString();
                if (length > 1) {
                    value = objs[1].toString();
                }
            }
        } else {
            parameter = valueObj.toString();
        }
        JSONWriter jsonWriter = new JSONWriter(appendable);
        try {
            jsonWriter.object();
            jsonWriter.key("error");
            jsonWriter.object();
            jsonWriter.key("key");
            jsonWriter.value(errorKey);
            jsonWriter.key("parameter");
            jsonWriter.value(parameter);
            if (value != null) {
                jsonWriter.key("value");
                jsonWriter.value(value);
            }
            if (commandMessage.hasMultiError()) {
                addErrorList(jsonWriter, commandMessage.getMultiErrorList());
            }
            jsonWriter.endObject();
            writeWarnings(jsonWriter);
            jsonWriter.endObject();
        } catch (JSONException je) {
            Throwable throwable = je.getCause();
            if (throwable == null) {
                throw new ShouldNotOccurException(je.getMessage(), je);
            } else {
                if (throwable instanceof IOException) {
                    throw ((IOException) throwable);
                } else {
                    throw new ShouldNotOccurException(throwable.getMessage(), (Exception) throwable);
                }
            }
        }
    }

    private static void addErrorList(JSONWriter jsonWriter, MessageList errorList) throws JSONException {
        int count = errorList.getMessageCount();
        jsonWriter.key("array");
        jsonWriter.array();
        for (int i = 0; i < count; i++) {
            Message errorMessage = errorList.getMessage(i);
            jsonWriter.object();
            jsonWriter.key("key");
            jsonWriter.value(errorMessage.getMessageKey());
            Object value = errorMessage.getMessageValue();
            if (value != null) {
                jsonWriter.key("value");
                jsonWriter.value(value.toString());
            }
            jsonWriter.endObject();
        }
        jsonWriter.endArray();
    }

}
