/* SctServer - Copyright (c) 2013-2014 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.sctserver.request.json;

import fr.exemole.sctserver.json.JsonUtils;
import fr.exemole.sctserver.api.ScrutariSession;
import java.io.IOException;
import net.mapeadores.util.exceptions.ShouldNotOccurException;
import net.mapeadores.util.json.JSONException;
import net.mapeadores.util.json.JSONWriter;
import net.mapeadores.util.localisation.Lang;
import net.mapeadores.util.localisation.MessageLocalisation;
import net.mapeadores.util.text.StringUtils;


/**
 *
 * @author Vincent Calame
 */
public class LangJsonProducer extends AbstractJsonProducer {

    private final ScrutariSession scrutariSession;
    private final int version;
    private final Lang langUi;
    private final Lang[] langArray;

    public LangJsonProducer(ScrutariSession scrutariSession, int version, Lang langUi, Lang[] langArray) {
        if (langUi == null) {
            throw new IllegalArgumentException("langUi is null");
        }
        this.scrutariSession = scrutariSession;
        this.version = version;
        this.langUi = langUi;
        this.langArray = langArray;
    }

    @Override
    public void writeJson(Appendable appendable) throws IOException {
        MessageLocalisation messageLocalisation = JsonUtils.getMessageLocalisation(scrutariSession.getSctServer(), langUi);
        JSONWriter jsonWriter = new JSONWriter(appendable);
        try {
            jsonWriter.object();
            jsonWriter.key("langMap");
            jsonWriter.object();
            if (langArray != null) {
                int count = langArray.length;
                for (int i = 0; i < count; i++) {
                    Lang langInteger = langArray[i];
                    String code = langInteger.toString();
                    jsonWriter.key(code);
                    String value = messageLocalisation.toString(code, null);
                    if (value != null) {
                        value = StringUtils.getFirstPart(value);
                    } else {
                        value = code;
                    }
                    jsonWriter.value(value);
                }
            }
            jsonWriter.endObject();
            writeWarnings(jsonWriter);
            jsonWriter.endObject();
        } catch (JSONException je) {
            Throwable throwable = je.getCause();
            if (throwable == null) {
                throw new ShouldNotOccurException(je.getMessage(), je);
            } else {
                if (throwable instanceof IOException) {
                    throw ((IOException) throwable);
                } else {
                    throw new ShouldNotOccurException(throwable.getMessage(), (Exception) throwable);
                }
            }
        }
    }

}
