/* SctServer - Copyright (c) 2010-2014 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.sctserver.request.json;

import fr.exemole.sctserver.json.JsonUtils;
import java.io.IOException;
import net.mapeadores.util.exceptions.ShouldNotOccurException;
import net.mapeadores.util.json.JSONException;
import net.mapeadores.util.json.JSONWriter;
import net.mapeadores.util.localisation.Lang;
import net.mapeadores.util.text.Label;
import net.scrutari.data.MotcleData;
import net.scrutari.db.api.DataAccess;
import net.scrutari.db.api.ScrutariDB;
import net.scrutari.searchengine.api.result.InLangOccurrenceList;
import net.scrutari.searchengine.api.result.MotcleSearchResult;
import net.scrutari.searchengine.api.result.MotcleSearchResultInfo;


/**
 *
 * @author Vincent Calame
 */
public class MotcleSearchResultJsonProducer extends AbstractJsonProducer {

    private final int version;
    private final ScrutariDB scrutariDB;
    private final MotcleSearchResult motcleSearchResult;
    private final Lang lang;
    private final int limit;

    public MotcleSearchResultJsonProducer(int version, MotcleSearchResult motcleSearchResult, ScrutariDB scrutariDB, Lang lang, int limit) {
        this.version = version;
        this.scrutariDB = scrutariDB;
        this.motcleSearchResult = motcleSearchResult;
        this.lang = lang;
        this.limit = limit;
    }

    @Override
    public void writeJson(Appendable appendable) throws IOException {
        DataAccess dataAccess = scrutariDB.openDataAccess();
        JSONWriter jsonWriter = new JSONWriter(appendable);
        try {
            jsonWriter.object();
            jsonWriter.key("motcleSearchResult");
            jsonWriter.object();
            int count = motcleSearchResult.getMotcleCount();
            if (limit != -1) {
                count = Math.max(count, limit);
            }
            jsonWriter.key("count");
            jsonWriter.value(count);
            if (count > 0) {
                jsonWriter.key("infoArray");
                jsonWriter.array();
                for (int i = 0; i < count; i++) {
                    MotcleSearchResultInfo info = motcleSearchResult.getMotcleResult(i);
                    int code = info.getCode();
                    MotcleData motcleData = dataAccess.getMotcleDataByCode(code);
                    InLangOccurrenceList occurrenceList = info.getInLangOccurrenceListByLang(lang);
                    Label libelle = motcleData.getLibelleByLang(lang);
                    jsonWriter.object();
                    jsonWriter.key("cdmtc");
                    jsonWriter.value(code);
                    jsonWriter.key("cdths");
                    jsonWriter.value(motcleData.getCodeThesaurus());
                    if ((libelle != null) && (occurrenceList != null)) {
                        String lib = libelle.getLabelString();
                        jsonWriter.key("lib");
                        jsonWriter.value(lib);
                        jsonWriter.key("mlib");
                        JsonUtils.writeMLib(jsonWriter, lib, occurrenceList);
                    }
                    jsonWriter.endObject();
                }
                jsonWriter.endArray();
            }
            jsonWriter.endObject();
            writeWarnings(jsonWriter);
            jsonWriter.endObject();
        } catch (JSONException je) {
            Throwable throwable = je.getCause();
            if (throwable == null) {
                throw new ShouldNotOccurException(je.getMessage(), je);
            } else {
                if (throwable instanceof IOException) {
                    throw ((IOException) throwable);
                } else {
                    throw new ShouldNotOccurException(throwable.getMessage(), (Exception) throwable);
                }
            }
        }
        dataAccess.close();
    }

}
