/* SctServer - Copyright (c) 2014-2016 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.sctserver.tools.collect;

import fr.exemole.sctserver.api.collect.ScrutariSourceDef;
import java.net.URL;
import java.util.ArrayList;
import java.util.LinkedHashSet;
import java.util.List;
import java.util.Set;
import net.mapeadores.util.text.StringUtils;
import net.scrutari.datauri.BaseURI;


/**
 *
 * @author Vincent Calame
 */
public class ScrutariSourceDefBuilder {

    private final String name;
    private BaseURI mainBaseURI;
    private String baseIcon;
    private final Set<BaseURI> aliasSet = new LinkedHashSet<BaseURI>();
    private final List<UrlInfo> urlInfoList = new ArrayList<UrlInfo>();


    public ScrutariSourceDefBuilder(String name) {
        if (!StringUtils.isTechnicalName(name, true)) {
            throw new IllegalArgumentException("Not a technical name: " + name);
        }
        this.name = name;
    }

    public void addScrutariDataUrl(URL url, int frequency) {
        if (url == null) {
            throw new IllegalArgumentException("url is null");
        }
        if (frequency < 1) {
            frequency = 1;
        }
        urlInfoList.add(new UrlInfo(ScrutariSourceDef.SCRUTARIDATA_TYPE, url, frequency));
    }

    public void addInfoUrl(URL url) {
        if (url == null) {
            throw new IllegalArgumentException("url is null");
        }
        urlInfoList.add(new UrlInfo(ScrutariSourceDef.INFO_TYPE, url, 1));
    }

    public void setMainBaseURI(BaseURI mainBaseURI) {
        this.mainBaseURI = mainBaseURI;
    }

    public void addAliasBaseURI(BaseURI baseURI) {
        aliasSet.add(baseURI);
    }

    public void setBaseIcon(String baseIcon) {
        if ((baseIcon != null) && (baseIcon.length() == 0)) {
            this.baseIcon = null;
        } else {
            this.baseIcon = baseIcon;
        }
    }

    public ScrutariSourceDef toScrutariSourceDef() {
        if (urlInfoList.isEmpty()) {
            throw new IllegalStateException("url is not initialized");
        }
        BaseURI[] aliasArray = aliasSet.toArray(new BaseURI[aliasSet.size()]);
        UrlInfo[] urlInfoArray = urlInfoList.toArray(new UrlInfo[urlInfoList.size()]);
        return new InternalScrutariSourceDef(name, urlInfoArray, mainBaseURI, aliasArray, baseIcon);
    }


    private static class UrlInfo {

        private final short urlType;
        private final URL url;
        private final int frequency;

        private UrlInfo(short urlType, URL url, int frequency) {
            this.urlType = urlType;
            this.url = url;
            this.frequency = frequency;
        }

    }


    private static class InternalScrutariSourceDef implements ScrutariSourceDef {

        private final String name;
        private final UrlInfo[] urlInfoArray;
        private final BaseURI mainBaseURI;
        private final BaseURI[] aliasArray;
        private final String baseIcon;

        private InternalScrutariSourceDef(String name, UrlInfo[] urlInfoArray, BaseURI mainBaseURI, BaseURI[] aliasArray, String baseIcon) {
            this.name = name;
            this.urlInfoArray = urlInfoArray;
            this.mainBaseURI = mainBaseURI;
            this.aliasArray = aliasArray;
            this.baseIcon = baseIcon;
        }

        @Override
        public String getName() {
            return name;
        }


        @Override
        public int getUrlCount() {
            return urlInfoArray.length;
        }

        @Override
        public short getUrlType(int i) {
            return urlInfoArray[i].urlType;
        }

        @Override
        public URL getUrl(int i) {
            return urlInfoArray[i].url;
        }

        @Override
        public int getFrequency(int i) {
            return urlInfoArray[i].frequency;
        }

        @Override
        public BaseURI getMainBaseURI() {
            return mainBaseURI;
        }

        @Override
        public int getAliasCount() {
            return aliasArray.length;
        }

        @Override
        public BaseURI getAlias(int i) {
            return aliasArray[i];
        }

        @Override
        public String getBaseIcon() {
            return baseIcon;
        }

    }

}
