/* SctServer - Copyright (c) 2013-2014 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.sctserver.tools.collect;

import fr.exemole.sctserver.api.collect.ScrutariSource;
import fr.exemole.sctserver.api.collect.ScrutariSourceDef;
import fr.exemole.sctserver.api.collect.ScrutariSourceList;
import java.util.HashMap;
import java.util.Map;
import net.mapeadores.util.primitives.DateInteger;
import net.mapeadores.util.primitives.DateIntegerException;
import net.mapeadores.util.text.StringUtils;


/**
 *
 * @author Vincent Calame
 */
public final class ScrutariSourceStorage {

    private ScrutariSourceStorage() {
    }

    public static String toString(ScrutariSourceList scrutariSourceList) {
        StringBuilder buf = new StringBuilder();
        int count = scrutariSourceList.getScrutariSourceCount();
        for (int i = 0; i < count; i++) {
            ScrutariSource scrutariSource = scrutariSourceList.getScrutariSource(i);
            ScrutariSourceDef scrutariSourceDef = scrutariSource.getScrutariSourceDef();
            buf.append(scrutariSourceDef.getName());
            buf.append("    ");
            short state = scrutariSource.getState();
            buf.append(CollectUtils.stateToString(state));
            buf.append("    ");
            int lastUpdate = scrutariSource.getLastUpdate();
            if (lastUpdate == DateInteger.NULL) {
                buf.append("NULL");
            } else {
                buf.append(DateInteger.toString(lastUpdate));
            }
            buf.append('\n');
        }
        return buf.toString();
    }

    public static Map<String, Status> fromString(String s) {
        Map<String, Status> map = new HashMap<String, Status>();
        String[] tokens = StringUtils.getLineTokens(s, StringUtils.EMPTY_EXCLUDE);
        int length = tokens.length;
        for (int i = 0; i < length; i++) {
            String ligne = tokens[i];
            String[] subtokens = StringUtils.getTechnicalTokens(ligne, true);
            int subtokenLength = subtokens.length;
            if (subtokenLength < 2) {
                continue;
            }
            boolean error = false;
            String name = subtokens[0];
            short state = ScrutariSource.UNKNOWN_STATE;
            try {
                state = CollectUtils.stateToShort(subtokens[1]);
            } catch (IllegalArgumentException iae) {
                error = true;
            }
            int lastUpdate = DateInteger.NULL;
            if (subtokenLength > 2) {
                if (!subtokens[2].equals("NULL")) {
                    try {
                        lastUpdate = DateInteger.parse(subtokens[2]);
                    } catch (DateIntegerException die) {
                        error = true;
                    }
                }
            }
            if (!error) {
                map.put(name, new Status(name, state, lastUpdate));
            }
        }
        return map;
    }


    public static class Status {

        private final String name;
        private final short state;
        private final int lastUpdate;

        private Status(String name, short state, int lastUpdate) {
            this.name = name;
            this.state = state;
            this.lastUpdate = lastUpdate;
        }

        public String getName() {
            return name;
        }

        public short getState() {
            return state;
        }

        public int getLastUpdate() {
            return lastUpdate;
        }

    }

}
