/* SctServer - Copyright (c) 2011-2016 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.sctserver.tools.conf.dom;

import fr.exemole.sctserver.tools.ErrorUtils;
import net.mapeadores.util.logging.LogUtils;
import net.mapeadores.util.logging.MessageHandler;
import net.mapeadores.util.xml.XMLUtils;
import net.scrutari.datauri.ScrutariDataURI;
import net.scrutari.datauri.URIParseException;
import net.scrutari.datauri.URIParser;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;


/**
 *
 * @author Vincent Calame
 */
public class UriCodesDOMReader {

    private final MessageHandler errorHandler;
    private final Builder builder;

    public UriCodesDOMReader(Builder builder, MessageHandler errorHandler) {
        this.errorHandler = errorHandler;
        this.builder = builder;
    }

    public void readUriCodes(Element element_xml, String xpath) {
        xpath = xpath + "/" + element_xml.getTagName();
        NodeList liste = element_xml.getChildNodes();
        for (int i = 0; i < liste.getLength(); i++) {
            Node nd = liste.item(i);
            if (nd.getNodeType() == Node.ELEMENT_NODE) {
                Element element = (Element) liste.item(i);
                String tagName = element.getTagName();
                if (tagName.equals("uri")) {
                    addUri(element, xpath);
                } else {
                    ErrorUtils.unknownTagWarning(errorHandler, xpath);
                }
            }
        }
    }

    private void addUri(Element element_xml, String xpath) {
        xpath = xpath + "/" + element_xml.getTagName();
        String codeString = element_xml.getAttribute("code");
        if (codeString.length() == 0) {
            ErrorUtils.emptyAttributeError(errorHandler, xpath, "code");
            return;
        }
        xpath = xpath + "[@code='" + codeString + "']";
        int code = 0;
        try {
            code = Integer.parseInt(codeString);
            if (code < 1) {
                ErrorUtils.severe(errorHandler, "_ error.wrong.xml.attributevalue_withreason", LogUtils.toValue(xpath, "code", codeString, "code < 1"));
            }
        } catch (NumberFormatException nfe) {
            ErrorUtils.severe(errorHandler, "_ error.wrong.xml.attributevalue_integer", LogUtils.toValue(xpath, "code", codeString));
        }
        if (builder.containsCode(code)) {
            ErrorUtils.severe(errorHandler, "_ error.existing.xml.attributevalue", LogUtils.toValue(xpath, "code", code));
        } else {
            String uriString = XMLUtils.getData(element_xml);
            if (uriString.length() == 0) {
                ErrorUtils.emptyElementError(errorHandler, xpath);
            } else {
                try {
                    ScrutariDataURI scrutariDataURI = URIParser.parse(uriString);
                    if (builder.containsURI(scrutariDataURI)) {
                        ErrorUtils.severe(errorHandler, "_ error.existing.xml.tagvalue", LogUtils.toValue(xpath, uriString));
                    } else {
                        builder.addScrutariDataURI(code, scrutariDataURI);
                    }
                } catch (URIParseException uripe) {
                    ErrorUtils.severe(errorHandler, "_ error.wrong.xml.tagvalue_uri", LogUtils.toValue(xpath, uriString, uripe.getMessage()));
                }
            }
        }
    }


    public static interface Builder {

        public boolean containsCode(int code);

        public boolean containsURI(ScrutariDataURI scrutariDataURI);

        public void addScrutariDataURI(int code, ScrutariDataURI scrutariDataURI);

    }

}
