/* UtilLib - Copyright (c) 2011-2016 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.mapeadores.util.attr;

import java.util.LinkedHashMap;
import java.util.LinkedHashSet;
import java.util.Map;
import java.util.Set;


/**
 *
 * @author Vincent Calame
 */
public class ChangeAttributeHolderBuilder {

    private final Set<AttributeKey> removedKeySet = new LinkedHashSet<AttributeKey>();
    private final Map<AttributeKey, Attribute> attributeMap = new LinkedHashMap<AttributeKey, Attribute>();

    public ChangeAttributeHolderBuilder() {
    }

    public void putAttribute(Attribute attribute) {
        AttributeKey attributeKey = attribute.getAttributeKey();
        attributeMap.put(attributeKey, attribute);
        removedKeySet.remove(attributeKey);
    }

    public void putRemovedAttributeKey(AttributeKey attributeKey) {
        removedKeySet.add(attributeKey);
        attributeMap.remove(attributeKey);
    }

    public void merge(ChangeAttributeHolderBuilder otherChangeAttributeHolderBuilder) {
        for (AttributeKey otherAttributeKey : otherChangeAttributeHolderBuilder.removedKeySet) {
            if (!attributeMap.containsKey(otherAttributeKey)) {
                removedKeySet.add(otherAttributeKey);
            }
        }
        for (Map.Entry<AttributeKey, Attribute> entry : otherChangeAttributeHolderBuilder.attributeMap.entrySet()) {
            AttributeKey otherAttributeKey = entry.getKey();
            Attribute otherAttribute = entry.getValue();
            Attribute currentAttribute = attributeMap.get(otherAttributeKey);
            if (currentAttribute != null) {
                AttributeBuilder attributeBuilder = new AttributeBuilder(otherAttributeKey);
                int currentCount = currentAttribute.getValueCount();
                for (int i = 0; i < currentCount; i++) {
                    attributeBuilder.addValue(currentAttribute.getValue(i));
                }
                int otherCount = otherAttribute.getValueCount();
                for (int i = 0; i < otherCount; i++) {
                    attributeBuilder.addValue(otherAttribute.getValue(i));
                }
                attributeMap.put(otherAttributeKey, attributeBuilder.toAttribute());
            } else {
                attributeMap.put(otherAttributeKey, otherAttribute);
            }
            removedKeySet.remove(otherAttributeKey);
        }
    }

    public ChangeAttributeHolder toChangeAttributeHolder() {
        int removedSize = removedKeySet.size();
        AttributeKey[] removedAttributeKeyArray = new AttributeKey[removedSize];
        if (removedSize > 0) {
            int p = 0;
            for (AttributeKey attributeKey : removedKeySet) {
                removedAttributeKeyArray[p] = attributeKey;
                p++;
            }
        }
        int attributeSize = attributeMap.size();
        Attribute[] attributeArray = new Attribute[attributeSize];
        if (attributeSize > 0) {
            int p = 0;
            for (Attribute attribute : attributeMap.values()) {
                attributeArray[p] = attribute;
                p++;
            }
        }
        return new InternalChangeAttributeHolder(attributeMap, attributeArray, removedAttributeKeyArray);
    }

    public AttributeHolder toAttributeHolder() {
        int attributeSize = attributeMap.size();
        Attribute[] attributeArray = new Attribute[attributeSize];
        if (attributeSize > 0) {
            int p = 0;
            for (Attribute attribute : attributeMap.values()) {
                attributeArray[p] = attribute;
                p++;
            }
        }
        return new InternalAttributeHolder(attributeMap, attributeArray);
    }


    private static class InternalChangeAttributeHolder implements ChangeAttributeHolder {

        private final AttributeKey[] removedAttributeKeyArray;
        private final Attribute[] attributeArray;
        private final Map<AttributeKey, Attribute> attributeMap;

        private InternalChangeAttributeHolder(Map<AttributeKey, Attribute> attributeMap, Attribute[] attributeArray, AttributeKey[] removedAttributeKeyArray) {
            this.removedAttributeKeyArray = removedAttributeKeyArray;
            this.attributeArray = attributeArray;
            this.attributeMap = attributeMap;
        }

        @Override
        public Attribute getAttributeByKey(AttributeKey key) {
            return attributeMap.get(key);
        }

        @Override
        public int getAttributeCount() {
            return attributeArray.length;
        }

        @Override
        public Attribute getAttributeByIndex(int i) {
            return attributeArray[i];
        }

        @Override
        public int getRemovedAttributeCount() {
            return removedAttributeKeyArray.length;
        }

        @Override
        public AttributeKey getRemovedAttributeKey(int i) {
            return removedAttributeKeyArray[i];
        }

    }


    private static class InternalAttributeHolder implements AttributeHolder {

        private final Attribute[] attributeArray;
        private final Map<AttributeKey, Attribute> attributeMap;

        private InternalAttributeHolder(Map<AttributeKey, Attribute> attributeMap, Attribute[] attributeArray) {
            this.attributeArray = attributeArray;
            this.attributeMap = attributeMap;
        }

        @Override
        public Attribute getAttributeByKey(AttributeKey key) {
            return attributeMap.get(key);
        }

        @Override
        public int getAttributeCount() {
            return attributeArray.length;
        }

        @Override
        public Attribute getAttributeByIndex(int i) {
            return attributeArray[i];
        }

    }

}
