/* UtilLib - Copyright (c) 2010-2014 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.mapeadores.util.html;

import java.util.HashMap;
import java.util.Map;


/**
 *
 * @author Vincent Calame
 */
public class CommandBox {

    private String commandString;
    private String commandKey;
    private Map<String, String> hiddenAttributesMap;
    private String page;
    private String errorPage;
    private String formAction;
    private String commandFamily;
    private String target;
    private boolean multipart = false;

    public CommandBox() {
    }

    public CommandBox(CommandBox commandBox) {
        this.commandString = commandBox.commandString;
        this.commandKey = commandBox.commandKey;
        if (commandBox.hiddenAttributesMap != null) {
            this.hiddenAttributesMap = new HashMap<String, String>(commandBox.hiddenAttributesMap);
        }
        this.page = commandBox.page;
        this.errorPage = commandBox.errorPage;
        this.formAction = commandBox.formAction;
        this.commandFamily = commandBox.commandFamily;
        this.target = commandBox.target;
        this.multipart = commandBox.multipart;
    }

    private CommandBox(CommandBox commandBox, String commandString, String commandKey) {
        this.commandString = commandString;
        this.commandKey = commandKey;
        if (commandBox.hiddenAttributesMap != null) {
            this.hiddenAttributesMap = new HashMap<String, String>(commandBox.hiddenAttributesMap);
        }
        this.page = commandBox.page;
        this.errorPage = commandBox.errorPage;
        this.formAction = commandBox.formAction;
        this.commandFamily = commandBox.commandFamily;
        this.target = commandBox.target;
        this.multipart = commandBox.multipart;
    }

    public CommandBox(String commandString, String commandKey) {
        this(commandString, commandKey, null);
    }

    public CommandBox(String commandString, String commandKey, String page) {
        this.commandString = commandString;
        this.commandKey = commandKey;
        this.page = page;
    }

    public CommandBox derive(String commandString, String commandKey) {
        return new CommandBox(this, commandString, commandKey);
    }

    public String getCommandString() {
        return commandString;
    }

    public CommandBox setCommandString(String commandString) {
        this.commandString = commandString;
        return this;
    }

    public String getCommandKey() {
        return commandKey;
    }

    public CommandBox setCommandKey(String commandKey) {
        this.commandKey = commandKey;
        return this;
    }

    public Map<String, String> getHiddenAttributesMap() {
        return hiddenAttributesMap;
    }

    public CommandBox putHiddenAttribute(String name, String value) {
        if (hiddenAttributesMap == null) {
            hiddenAttributesMap = new HashMap<String, String>();
        }
        hiddenAttributesMap.put(name, value);
        return this;
    }

    public String getPage() {
        return page;
    }

    public CommandBox setPage(String page) {
        this.page = page;
        return this;
    }

    public String getErrorPage() {
        return errorPage;
    }

    public CommandBox setErrorPage(String errorPage) {
        this.errorPage = errorPage;
        return this;
    }

    public String getFormAction() {
        return formAction;
    }

    public CommandBox setFormAction(String formAction) {
        this.formAction = formAction;
        return this;
    }

    public String getCommandFamily() {
        return commandFamily;
    }

    public CommandBox setCommandFamily(String commandFamily) {
        this.commandFamily = commandFamily;
        return this;
    }

    public String getTarget() {
        return target;
    }

    public CommandBox setTarget(String target) {
        this.target = target;
        return this;
    }

    public boolean isMultipart() {
        return multipart;
    }

    public CommandBox setMultipart(boolean multipart) {
        this.multipart = multipart;
        return this;
    }

}
