/* UtilLib - Copyright (c) 2016 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.mapeadores.util.logging;

import java.util.ArrayList;
import java.util.List;
import net.mapeadores.util.localisation.LocalisationUtils;
import net.mapeadores.util.localisation.Message;
import net.mapeadores.util.localisation.MessageList;
import net.mapeadores.util.localisation.MessageListBuilder;


/**
 *
 * @author Vincent Calame
 */
public class CategorizedMessageListBuilder {

    private final List<String> categoryList = new ArrayList<String>();
    private final List<Message> messageList = new ArrayList<Message>();

    public CategorizedMessageListBuilder() {

    }

    public boolean isEmpty() {
        return (messageList.isEmpty());
    }

    public void clear() {
        categoryList.clear();
        messageList.clear();
    }

    public void add(String category, Message message) {
        if (category == null) {
            throw new IllegalArgumentException("category is null");
        }
        if (message == null) {
            throw new IllegalArgumentException("message is null");
        }
        categoryList.add(category);
        messageList.add(message);
    }

    public void add(String category, String messageKey, Object messageValue) {
        if (category == null) {
            throw new IllegalArgumentException("category is null");
        }
        if (messageKey == null) {
            throw new IllegalArgumentException("message is null");
        }
        categoryList.add(category);
        messageList.add(LocalisationUtils.toMessage(messageKey, messageValue));
    }


    public CategorizedMessageList toCategorizedMessageList() {
        int size = messageList.size();
        return new InternalCategorizedMessageList(categoryList.toArray(new String[size]), messageList.toArray(new Message[size]));
    }

    public MessageList toMessageList(String category) {
        MessageListBuilder builder = new MessageListBuilder();
        int size = messageList.size();
        for (int i = 0; i < size; i++) {
            if (categoryList.get(i).equals(category)) {
                builder.addMessage(messageList.get(i));
            }
        }
        return builder.toMessageList();
    }


    private static class InternalCategorizedMessageList implements CategorizedMessageList {

        private final String[] categoryArray;
        private final Message[] messageArray;

        private InternalCategorizedMessageList(String[] categoryArray, Message[] messageArray) {
            this.categoryArray = categoryArray;
            this.messageArray = messageArray;
        }

        @Override
        public int getMessageCount() {
            return messageArray.length;
        }

        @Override
        public Message getMessage(int i) {
            return messageArray[i];
        }

        @Override
        public String getCategory(int i) {
            return categoryArray[i];
        }

    }

}
