/* UtilLib - Copyright (c) 2008-2016 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.mapeadores.util.logging;

import net.mapeadores.util.localisation.Message;
import net.mapeadores.util.localisation.MessageList;


/**
 *
 * @author Vincent Calame
 */
public class CommandMessage implements Message {

    private String messageKey;
    private Object messageValue;
    private MessageLog messageLog;
    private MessageList multiErrorList;
    private boolean errorMessage = false;

    private CommandMessage(String messageKey, Object messageValue) {
        if (messageKey == null) {
            throw new IllegalArgumentException("messageKey is null");
        }
        this.messageKey = messageKey;
        this.messageValue = messageValue;
    }

    private void initIsErrorMessage(boolean errorMessage) {
        this.errorMessage = errorMessage;
    }

    private void initMultiErrorList(MessageList multiErrorList) {
        this.multiErrorList = multiErrorList;
    }

    private void initMessageLog(MessageLog messageLog) {
        this.messageLog = messageLog;
    }

    @Override
    public String getMessageKey() {
        return messageKey;
    }

    @Override
    public Object getMessageValue() {
        return messageValue;
    }

    public boolean isErrorMessage() {
        return errorMessage;
    }

    public boolean hasMultiError() {
        return (multiErrorList != null);
    }

    public MessageList getMultiErrorList() {
        return multiErrorList;
    }

    public boolean hasLog() {
        return ((messageLog != null) && (messageLog.getLogGroupCount() > 0));
    }

    public MessageLog getMessageLog() {
        return messageLog;
    }

    public static CommandMessage error(String messageKey, Object messageValue) {
        CommandMessage commandMessage = new CommandMessage(messageKey, messageValue);
        commandMessage.initIsErrorMessage(true);
        return commandMessage;
    }

    public static CommandMessage error(Message message) {
        CommandMessage commandMessage = new CommandMessage(message.getMessageKey(), message.getMessageValue());
        commandMessage.initIsErrorMessage(true);
        return commandMessage;
    }

    public static CommandMessage error(String messageKey, Object messageValue, MessageList multiErrorList) {
        CommandMessage commandMessage = new CommandMessage(messageKey, messageValue);
        commandMessage.initIsErrorMessage(true);
        commandMessage.initMultiErrorList(multiErrorList);
        return commandMessage;
    }

    public static CommandMessage done(String messageKey, Object messageValue) {
        CommandMessage commandMessage = new CommandMessage(messageKey, messageValue);
        return commandMessage;
    }

    public static CommandMessage done(Message message) {
        CommandMessage commandMessage = new CommandMessage(message.getMessageKey(), message.getMessageValue());
        return commandMessage;
    }

    public static CommandMessage done(String messageKey, Object messageValue, MessageLog processMessageLog) {
        CommandMessage commandMessage = new CommandMessage(messageKey, messageValue);
        commandMessage.initMessageLog(processMessageLog);
        return commandMessage;
    }

}
