/* UtilLib - Copyright (c) 2012-2016 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.mapeadores.util.logging;

import java.io.IOException;
import java.io.StringReader;
import javax.xml.parsers.DocumentBuilder;
import net.mapeadores.util.exceptions.ShouldNotOccurException;
import net.mapeadores.util.html.HA;
import net.mapeadores.util.html.HtmlPrinter;
import net.mapeadores.util.instruction.InstructionErrorHandler;
import net.mapeadores.util.json.JSONException;
import net.mapeadores.util.json.JSONWriter;
import net.mapeadores.util.localisation.LocalisationUtils;
import net.mapeadores.util.localisation.Message;
import net.mapeadores.util.xml.AppendableXMLWriter;
import net.mapeadores.util.xml.DOMUtils;
import net.mapeadores.util.xml.XMLUtils;
import org.w3c.dom.Document;
import org.xml.sax.InputSource;
import org.xml.sax.SAXException;


/**
 *
 * @author Vincent Calame
 */
public class LogUtils {

    public final static CategorizedMessageList EMPTY_CATEGORIZEDMESSAGELIST = new EmptyCategorizedMessageList();
    public final static MessageLog EMPTY_MESSAGELOG = new EmptyMessageLog();
    public final static MultiMessageHandler NULL_MULTIMESSAGEHANDLER = new NullMultiMessageHandler();
    public final static LineMessageHandler NULL_LINEMESSAGEHANDLER = new NullLineMessageHandler();

    private LogUtils() {
    }

    public static String[] toValue(Object s1) {
        String[] array = new String[1];
        array[0] = toString(s1);
        return array;
    }

    public static String[] toValue(Object s1, Object s2) {
        String[] array = new String[2];
        array[0] = toString(s1);
        array[1] = toString(s2);
        return array;
    }

    public static String[] toValue(Object s1, Object s2, Object s3) {
        String[] array = new String[3];
        array[0] = toString(s1);
        array[1] = toString(s2);
        array[2] = toString(s3);
        return array;
    }

    public static String[] toValue(Object s1, Object s2, Object s3, Object s4) {
        String[] array = new String[4];
        array[0] = toString(s1);
        array[1] = toString(s2);
        array[2] = toString(s3);
        array[3] = toString(s4);
        return array;
    }

    public static String[] toValue(Object s1, Object s2, Object s3, Object s4, Object s5) {
        String[] array = new String[5];
        array[0] = toString(s1);
        array[1] = toString(s2);
        array[2] = toString(s3);
        array[3] = toString(s4);
        array[4] = toString(s5);
        return array;
    }

    public static String[] toValue(Object s1, Object s2, Object s3, Object s4, Object s5, Object s6) {
        String[] array = new String[6];
        array[0] = toString(s1);
        array[1] = toString(s2);
        array[2] = toString(s3);
        array[3] = toString(s4);
        array[4] = toString(s5);
        array[5] = toString(s6);
        return array;
    }

    public static String[] toValue(Object s1, Object s2, Object s3, Object s4, Object s5, Object s6, Object s7) {
        String[] array = new String[7];
        array[0] = toString(s1);
        array[1] = toString(s2);
        array[2] = toString(s3);
        array[3] = toString(s4);
        array[4] = toString(s5);
        array[5] = toString(s6);
        array[6] = toString(s7);
        return array;
    }

    public static String[] toValue(Object s1, Object s2, Object s3, Object s4, Object s5, Object s6, Object s7, Object s8) {
        String[] array = new String[8];
        array[0] = toString(s1);
        array[1] = toString(s2);
        array[2] = toString(s3);
        array[3] = toString(s4);
        array[4] = toString(s5);
        array[5] = toString(s6);
        array[6] = toString(s7);
        array[7] = toString(s8);
        return array;
    }

    public static String[] toValue(Object s1, Object s2, Object s3, Object s4, Object s5, Object s6, Object s7, Object s8, Object s9) {
        String[] array = new String[9];
        array[0] = toString(s1);
        array[1] = toString(s2);
        array[2] = toString(s3);
        array[3] = toString(s4);
        array[4] = toString(s5);
        array[5] = toString(s6);
        array[6] = toString(s7);
        array[7] = toString(s8);
        array[8] = toString(s9);
        return array;
    }

    private static String toString(Object obj) {
        if (obj == null) {
            return "";
        } else {
            return obj.toString();
        }
    }

    public static String toString(String processURI, CategorizedMessageList messageList) {
        StringBuilder buf = new StringBuilder();
        try {
            append(buf, processURI, messageList);
        } catch (IOException ioe) {
        }
        return buf.toString();
    }

    public static String toString(MessageLog messageLog) {
        int groupCount = messageLog.getLogGroupCount();
        StringBuilder buf = new StringBuilder();
        try {
            for (int i = 0; i < groupCount; i++) {
                MessageLog.LogGroup group = messageLog.getLogGroup(i);
                append(buf, group.getURI(), group.getList());
            }
        } catch (IOException ioe) {
        }
        return buf.toString();
    }

    public static String toXmlString(MessageLog messageLog) {
        StringBuilder buf = new StringBuilder();
        try {
            AppendableXMLWriter xmlWriter = XMLUtils.getXMLWriter(buf);
            XMLUtils.appendXMLDeclaration(xmlWriter);
            MessageXMLPart messageXMLPart = new MessageXMLPart(xmlWriter);
            messageXMLPart.addLogs(messageLog);
        } catch (IOException ioe) {

        }
        return buf.toString();
    }

    public static String toXmlString(String uri, CategorizedMessageList processMessageList) {
        StringBuilder buf = new StringBuilder();
        try {
            AppendableXMLWriter xmlWriter = XMLUtils.getXMLWriter(buf);
            XMLUtils.appendXMLDeclaration(xmlWriter);
            MessageXMLPart messageXMLPart = new MessageXMLPart(xmlWriter);
            messageXMLPart.addLogGroup(uri, processMessageList);
        } catch (IOException ioe) {

        }
        return buf.toString();
    }


    public static CategorizedMessageList readProcessMessageList(String xmlString) {
        SimpleMessageHandler simpleHandler = new SimpleMessageHandler();
        DocumentBuilder docBuilder = DOMUtils.newDocumentBuilder();
        try {
            Document document = docBuilder.parse(new InputSource(new StringReader(xmlString)));
            MessageDOMReader reader = new MessageDOMReader(simpleHandler);
            return reader.readList(document.getDocumentElement(), "");
        } catch (SAXException saxe) {
            simpleHandler.addMessage("LOG XML FORMAT", LocalisationUtils.toMessage("_ error.exception.xml.sax", saxe.getMessage()));
            return simpleHandler.toList();
        } catch (IOException ioe) {
            throw new ShouldNotOccurException(ioe);
        }
    }

    public static MessageLog readProcessMessageLog(String xmlString) {
        SimpleMessageHandler simpleHandler = new SimpleMessageHandler();
        MessageLogBuilder logBuilder = new MessageLogBuilder();
        DocumentBuilder docBuilder = DOMUtils.newDocumentBuilder();
        try {
            Document document = docBuilder.parse(new InputSource(new StringReader(xmlString)));
            MessageDOMReader reader = new MessageDOMReader(simpleHandler);
            reader.readProcessMessageLog(logBuilder, document.getDocumentElement(), "");
        } catch (SAXException saxe) {
            logBuilder.setCurrentURI("logfile");
            logBuilder.addMessage("LOG XML FORMAT", LocalisationUtils.toMessage("_ error.exception.xml.sax", saxe.getMessage()));
        } catch (IOException ioe) {
            throw new ShouldNotOccurException(ioe);
        }
        return logBuilder.toMessageLog();
    }

    public static void printHtml(HtmlPrinter htmlPrinter, MessageLog messageLog) {
        int groupCount = messageLog.getLogGroupCount();
        htmlPrinter.startUl();
        for (int i = 0; i < groupCount; i++) {
            MessageLog.LogGroup group = messageLog.getLogGroup(i);
            CategorizedMessageList messageList = group.getList();
            int messageCount = messageList.getMessageCount();
            if (messageCount > 0) {
                htmlPrinter.startLi();
                htmlPrinter.startP();
                htmlPrinter.startCode(HA.classes("cm-link"));
                htmlPrinter.escape(group.getURI());
                htmlPrinter.endCode();
                htmlPrinter.endP();
                htmlPrinter.startUl();
                for (int j = 0; j < messageCount; j++) {
                    Message message = messageList.getMessage(j);
                    String category = messageList.getCategory(j);
                    htmlPrinter.startLi();
                    htmlPrinter.startP();
                    htmlPrinter.startCode(HA.classes("cm-qualifier"));
                    htmlPrinter.append("[");
                    htmlPrinter.escape(category);
                    htmlPrinter.append("]");
                    htmlPrinter.endCode();
                    htmlPrinter.space();
                    htmlPrinter.startSmall();
                    htmlPrinter.localize(message);
                    htmlPrinter.endSmall();
                    htmlPrinter.endP();
                    htmlPrinter.endLi();
                }
                htmlPrinter.endUl();
                htmlPrinter.endLi();
            }
        }
        htmlPrinter.endUl();
    }

    public static void writeJsonValue(JSONWriter jsonWriter, MessageLog messageLog) throws JSONException {
        int groupCount = messageLog.getLogGroupCount();
        jsonWriter.array();
        for (int i = 0; i < groupCount; i++) {
            MessageLog.LogGroup group = messageLog.getLogGroup(i);
            CategorizedMessageList messageList = group.getList();
            int messageCount = messageList.getMessageCount();
            if (messageCount > 0) {
                jsonWriter.object();
                jsonWriter.key("uri");
                jsonWriter.value(group.getURI());
                jsonWriter.key("messageArray");
                jsonWriter.array();
                for (int j = 0; j < messageCount; j++) {
                    Message message = messageList.getMessage(j);
                    String category = messageList.getCategory(j);
                    jsonWriter.object();
                    jsonWriter.key("category");
                    jsonWriter.value(category);
                    jsonWriter.key("key");
                    jsonWriter.value(message.getMessageKey());
                    Object value = message.getMessageValue();
                    if (value != null) {
                        jsonWriter.key("valueArray");
                        jsonWriter.array();
                        if (value instanceof Object[]) {
                            Object[] array = (Object[]) value;
                            for (Object obj : array) {
                                jsonWriter.value(obj.toString());
                            }
                        } else {
                            jsonWriter.value(value.toString());
                        }
                        jsonWriter.endArray();
                    }
                    jsonWriter.endObject();
                }
                jsonWriter.endArray();
                jsonWriter.endObject();
            }
        }
        jsonWriter.endArray();
    }

    public static void append(Appendable buf, String processURI, CategorizedMessageList messageList) throws IOException {
        int count = messageList.getMessageCount();
        if (count == 0) {
            return;
        }
        boolean withTabs = false;
        if (processURI != null) {
            buf.append("[[");
            buf.append(processURI);
            buf.append("]]");
            buf.append("\n");
            withTabs = true;
        }
        for (int j = 0; j < count; j++) {
            Message message = messageList.getMessage(j);
            String category = messageList.getCategory(j);
            if (withTabs) {
                buf.append("      ");
            }
            buf.append("[");
            buf.append(category);
            buf.append("] ");
            buf.append(message.getMessageKey());
            Object value = message.getMessageValue();
            if (value != null) {
                buf.append(" = ");
                if (value instanceof Object[]) {
                    Object[] array = (Object[]) value;
                    int length = array.length;
                    for (int i = 0; i < length; i++) {
                        if (i > 0) {
                            buf.append(" | ");
                        }
                        buf.append(array[i].toString());
                    }
                } else {
                    buf.append(value.toString());
                }
            }
            buf.append("\n");
        }
    }

    public static InstructionErrorHandler encapsulate(LineMessageHandler lineMessageHandler, String category, int lineNumber) {
        return new LineInstructionErrorHandler(lineMessageHandler, category, lineNumber);
    }


    private static class EmptyCategorizedMessageList implements CategorizedMessageList {

        private EmptyCategorizedMessageList() {
        }

        @Override
        public int getMessageCount() {
            return 0;
        }

        @Override
        public Message getMessage(int i) {
            throw new IndexOutOfBoundsException();
        }

        @Override
        public String getCategory(int i) {
            throw new IndexOutOfBoundsException();
        }

    }


    private static class EmptyMessageLog implements MessageLog {

        private EmptyMessageLog() {
        }

        @Override
        public int getLogGroupCount() {
            return 0;
        }

        @Override
        public LogGroup getLogGroup(int i) {
            throw new IndexOutOfBoundsException();
        }

    }


    private static class NullMultiMessageHandler implements MultiMessageHandler {

        private NullMultiMessageHandler() {
        }

        @Override
        public void addMessage(String category, Message message) {
        }

        @Override
        public void setCurrentURI(String uri) {
        }

    }


    private static class NullLineMessageHandler implements LineMessageHandler {

        private NullLineMessageHandler() {
        }

        @Override
        public void addMessage(int lineNumber, String category, Message message) {
        }


    }


    private static class LineInstructionErrorHandler implements InstructionErrorHandler {

        private final String category;
        private final LineMessageHandler lineMessageHandler;
        private final int lineNumber;

        private LineInstructionErrorHandler(LineMessageHandler lineMessageHandler, String category, int lineNumber) {
            this.lineMessageHandler = lineMessageHandler;
            this.category = category;
            this.lineNumber = lineNumber;
        }

        @Override
        public void invalidAsciiCharacterError(String part, int row, int col) {
            addMessage("_ error.wrong.character_notascii", part, row, col);
        }

        @Override
        public void invalidEndCharacterError(String part, int row, int col) {
            addMessage("_ error.wrong.character_end", part, row, col);
        }

        @Override
        public void invalidSeparatorCharacterError(String part, int row, int col) {
            addMessage("_ error.wrong.character_separator", part, row, col);
        }

        private void addMessage(String messageKey, String part, int row, int col) {
            lineMessageHandler.addMessage(lineNumber, category, LocalisationUtils.toMessage(messageKey, LogUtils.toValue(part, row, col)));
        }

    }

}
