/* UtilLib - Copyright (c) 2015-2016 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.mapeadores.util.logging;

import java.io.IOException;
import net.mapeadores.util.localisation.Message;
import net.mapeadores.util.xml.XMLPart;
import net.mapeadores.util.xml.XMLWriter;


/**
 *
 * @author Vincent Calame
 */
public class MessageXMLPart extends XMLPart {

    public MessageXMLPart(XMLWriter xmlWriter) {
        super(xmlWriter);
    }

    public void addLogs(MessageLog messageLog) throws IOException {
        openTag("logs");
        addLogGroups(messageLog);
        closeTag("logs");
    }

    public void addLogGroups(MessageLog messageLog) throws IOException {
        int count = messageLog.getLogGroupCount();
        for (int i = 0; i < count; i++) {
            MessageLog.LogGroup logGroup = messageLog.getLogGroup(i);
            addLogGroup(logGroup.getURI(), logGroup.getList());
        }
    }

    public void addLogGroup(String uri, CategorizedMessageList messageList) throws IOException {
        startOpenTag("log");
        addAttribute("uri", uri);
        endOpenTag();
        addMessageList(messageList);
        closeTag("log");
    }

    public void addMessageList(CategorizedMessageList messageList) throws IOException {
        int count = messageList.getMessageCount();
        for (int i = 0; i < count; i++) {
            addMessage(messageList.getMessage(i), messageList.getCategory(i));
        }
    }

    public void addMessage(Message message) throws IOException {
        addMessage(message, null);
    }

    private void addMessage(Message message, String category) throws IOException {
        startOpenTag("message");
        addAttribute("category", category);
        addAttribute("key", message.getMessageKey());
        Object values = message.getMessageValue();
        if (values == null) {
            closeEmptyTag();
        } else {
            endOpenTag();
            addValues(values);
            closeTag("message");
        }
    }

    private void addValues(Object value) throws IOException {
        if (value instanceof Object[]) {
            Object[] array = (Object[]) value;
            for (Object val : array) {
                addSimpleElement("value", val.toString());
            }
        } else {
            addSimpleElement("value", value.toString());
        }
    }

}
