/* UtilLib - Copyright (c) 2012-2016 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.mapeadores.util.logging;

import net.mapeadores.util.localisation.Message;
import net.mapeadores.util.localisation.MessageList;


/**
 *
 * @author Vincent Calame
 */
public class SimpleMessageHandler implements MessageHandler {

    private final CategorizedMessageListBuilder builder = new CategorizedMessageListBuilder();

    public SimpleMessageHandler() {
    }

    @Override
    public void addMessage(String category, Message message) {
        builder.add(category, message);
    }

    public boolean hasMessage() {
        return (!builder.isEmpty());
    }

    public CategorizedMessageList toList() {
        return builder.toCategorizedMessageList();
    }

    public MessageList filter(String category) {
        return builder.toMessageList(category);
    }

    @Override
    public String toString() {
        StringBuilder buf = new StringBuilder();
        CategorizedMessageList list = builder.toCategorizedMessageList();
        int count = list.getMessageCount();
        for (int j = 0; j < count; j++) {
            Message message = list.getMessage(j);
            if (buf.length() > 0) {
                buf.append("\n");
            }
            buf.append(message.getMessageKey());
            Object value = message.getMessageValue();
            if (value != null) {
                if (value instanceof Object[]) {
                    Object[] array = (Object[]) value;
                    int length = array.length;
                    for (int i = 0; i < length; i++) {
                        if (i > 0) {
                            buf.append(" / ");
                        } else {
                            buf.append(": ");
                        }
                        buf.append(array[i].toString());
                    }
                } else {
                    buf.append(": ");
                    buf.append(value.toString());
                }
            }
        }
        return buf.toString();
    }

}
