/* FichothequeLib_Tools - Copyright (c) 2016 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.mapeadores.util.misc;

import net.mapeadores.util.attr.Attribute;
import net.mapeadores.util.attr.AttributeHolder;
import net.mapeadores.util.attr.AttributeKey;
import net.mapeadores.util.attr.ChangeAttributeHolder;
import net.mapeadores.util.attr.ChangeAttributeHolderBuilder;
import net.mapeadores.util.localisation.Lang;
import net.mapeadores.util.text.ChangeLabelHolderBuilder;
import net.mapeadores.util.text.Label;
import net.mapeadores.util.text.LabelHolder;


/**
 *
 * @author Vincent Calame
 */
public class DefBuilder {

    private final ChangeAttributeHolderBuilder attributeHolderBuilder = new ChangeAttributeHolderBuilder();
    private final ChangeLabelHolderBuilder labelHolderBuilder = new ChangeLabelHolderBuilder();

    public DefBuilder() {

    }

    public void putLabel(Label label) {
        labelHolderBuilder.putLabel(label);
    }

    public void addAttributes(AttributeHolder attributeHolder) {
        int count = attributeHolder.getAttributeCount();
        for (int i = 0; i < count; i++) {
            attributeHolderBuilder.putAttribute(attributeHolder.getAttributeByIndex(i));
        }
    }

    public void changeAttributes(ChangeAttributeHolder changeAttributeHolder) {
        int count = changeAttributeHolder.getAttributeCount();
        for (int i = 0; i < count; i++) {
            attributeHolderBuilder.putAttribute(changeAttributeHolder.getAttributeByIndex(i));
        }
        int removedCount = changeAttributeHolder.getRemovedAttributeCount();
        for (int i = 0; i < removedCount; i++) {
            attributeHolderBuilder.putRemovedAttributeKey(changeAttributeHolder.getRemovedAttributeKey(i));
        }
    }

    protected LabelHolder toLabelHolder() {
        return labelHolderBuilder.toChangeLabelHolder();
    }

    protected AttributeHolder toAttributeHolder() {
        return attributeHolderBuilder.toAttributeHolder();
    }


    public static class Def implements LabelHolder, AttributeHolder {

        private final LabelHolder labelHolder;
        private final AttributeHolder attributeHolder;

        public Def(LabelHolder labelHolder, AttributeHolder attributeHolder) {
            this.labelHolder = labelHolder;
            this.attributeHolder = attributeHolder;
        }

        @Override
        public int getLabelCount() {
            return labelHolder.getLabelCount();
        }

        @Override
        public Label getLabelByIndex(int i) {
            return labelHolder.getLabelByIndex(i);
        }

        @Override
        public Label getLabelByLang(Lang lang) {
            return labelHolder.getLabelByLang(lang);
        }

        @Override
        public Attribute getAttributeByKey(AttributeKey key) {
            return attributeHolder.getAttributeByKey(key);
        }

        @Override
        public int getAttributeCount() {
            return attributeHolder.getAttributeCount();
        }

        @Override
        public Attribute getAttributeByIndex(int i) {
            return attributeHolder.getAttributeByIndex(i);
        }

    }

}
