/* UtilLib - Copyright (c) 2016 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.mapeadores.util.primitives;

import java.text.DecimalFormatSymbols;
import java.text.ParseException;
import net.mapeadores.util.text.StringUtils;


/**
 *
 * @author Vincent Calame
 */
public final class Amount {

    private final long moneyLong;
    private final int currencyInteger;

    public Amount(long moneyLong, int currencyInteger) {
        this.moneyLong = moneyLong;
        this.currencyInteger = currencyInteger;
    }

    public long getMoneyLong() {
        return moneyLong;
    }

    public int getCurrencyInteger() {
        return currencyInteger;
    }

    public String getCurrencyCode() {
        return CurrencyInteger.toString(currencyInteger);
    }

    public Decimal toDecimal(boolean avoidDecimal) {
        return MoneyLong.toDecimal(moneyLong, currencyInteger, avoidDecimal);
    }

    @Override
    public String toString() {
        return MoneyLong.toString(moneyLong, currencyInteger);
    }

    public String toLitteralString(DecimalFormatSymbols symbols, boolean avoidDecimal) {
        return MoneyLong.toLitteralString(moneyLong, currencyInteger, symbols, avoidDecimal);
    }

    public static Amount fromDecimal(Decimal decimal, int currencyInteger) {
        return new Amount(MoneyLong.toMoneyLong(decimal, currencyInteger), currencyInteger);
    }

    public static Amount parse(String s) throws ParseException {
        String[] result = MoneyUtils.splitMoney(s);
        if (result == null) {
            throw new ParseException(s, 0);
        }
        Decimal decimal;
        try {
            decimal = StringUtils.parseDecimal(result[0]);
        } catch (NumberFormatException nfe) {
            throw new ParseException(result[0], 0);
        }
        int currencyInteger;
        try {
            currencyInteger = CurrencyInteger.parse(result[1]);
        } catch (CurrencyIntegerException cie) {
            throw new ParseException(result[1], 0);
        }
        return fromDecimal(decimal, currencyInteger);
    }

}
